#import <Foundation/Foundation.h>
#import "TaskLauncher.h"

#import "xml_object.h"

//Mount point on inventory system (partition_)
//ip_address by network card (ip_address)

int main (int argc, const char * argv[])
{
    @autoreleasepool {
	
		NSUserDefaults *args = [NSUserDefaults standardUserDefaults];
		NSString *uuidTypeArg = [args stringForKey:@"uuidType"];
		NSString *uuidType;
		
		if (uuidTypeArg.length == 0 || uuidTypeArg == @"uuid") {
			uuidType = @"uuid";
		} else {
			uuidType = @"name";
		}
    
    TaskLauncher *taskLauncher = [[TaskLauncher alloc] init];
    NSString *sep = @"^";
    NSString *errorString;
    NSString *outFileName = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"mac.xml"];
    NSLog(@"%@", outFileName);
    if ([[NSFileManager defaultManager] fileExistsAtPath:outFileName])
        [[NSFileManager defaultManager] removeItemAtPath:outFileName error:NULL];
    
    [[NSFileManager defaultManager] createFileAtPath:outFileName contents:NULL attributes:NULL];
    
    NSFileHandle *outFile = [NSFileHandle fileHandleForWritingAtPath:outFileName];
    
#pragma mark Data and initial string
    
    //write current date and initial string
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
    [dateFormat setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    NSDate *now = [NSDate date];
    NSString *stringToWrite;
    NSString *theDate = [dateFormat stringFromDate:now];
    
    //get hostname
    NSData *outFileData = [taskLauncher launchTask: @"/bin/hostname" withArguments: [NSArray arrayWithObject:@"-s"]];
    NSString *outFileString = [[NSString alloc] initWithData:outFileData encoding:NSUTF8StringEncoding];
    NSString *hostName = [outFileString stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
    
    //get domain name
    outFileData = [taskLauncher launchTask: @"/bin/domainname"];
    outFileString = [[NSString alloc] initWithData:outFileData encoding:NSUTF8StringEncoding];
    NSString *net_domain = [outFileString stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
    
		NSString *system_name = [NSString stringWithFormat:@"%@.%@", hostName, net_domain];
    
    //whoami
    outFileData =[taskLauncher launchTask: @"/usr/bin/whoami"];
    
    outFileString = [[NSString alloc] initWithData:outFileData encoding:NSUTF8StringEncoding];
    NSString *net_user_name = [outFileString stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];

		//Fill hwArray and extract UUID    
    outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml",  @"SPHardwareDataType",nil]];
    
    NSMutableArray *hwArray = (NSMutableArray *)[NSPropertyListSerialization
													 propertyListFromData:outFileData
													 mutabilityOption:NSPropertyListMutableContainersAndLeaves
													 format:NULL
													 errorDescription:&errorString];
    
    NSDictionary *hwItems = [[[hwArray objectAtIndex:0] objectForKey:@"_items"] objectAtIndex:0];

		NSString *systemUUID = [hwItems objectForKey:@"platform_UUID"];
		
		NSString *system_uuid;
		if (uuidType == @"uuid") {
			system_uuid = systemUUID;
		}
		
		if (uuidType == @"name") {
			system_uuid = system_name;
		}
        
    NSString *systemSerial = [hwItems objectForKey:@"serial_number"];
    NSString *pcMemory = [hwItems objectForKey:@"physical_memory"]; //convert to number of bytes
    NSString *systemModel = [hwItems objectForKey:@"machine_model"];
    NSString *osBit = [hwItems objectForKey:@"cpu_type"];
    
    //Depending on the Processor, set osBit equal to 32 or 64 bits
    if ([osBit isEqualToString:@"Intel Core i5"] || [osBit isEqualToString:@"Intel Core i3"] || [osBit isEqualToString:@"Intel Core i7"] || [osBit isEqualToString:@"Quad-Core Intel Xeon"] || [osBit isEqualToString:@"Dual-Core Intel Xeon"] || [osBit isEqualToString:@"Intel Quad-Core Xeon"] || [osBit isEqualToString:@"Intel Core 2 Duo"]) {
        osBit = @"64";
    } else if ([osBit isEqualToString:@"Intel Core Solo"] || [osBit isEqualToString:@"Intel Core Duo"]) {
        osBit = @"32";
    } else { //default 64 bits because many, if not all recent models have 64 bits
        osBit = @"64";
    }
    
    //Convert from Memory in GB to Bytes(8bits) or should it be to bits?
    NSArray *pcMemList = [pcMemory componentsSeparatedByString:@" "];
    int pcMem = [pcMemList[0] intValue];
    pcMem = pcMem * 1048576;
    NSString *pcMemString = [NSString stringWithFormat:@"%d", pcMem];
    
#pragma mark OS (system) info
    
    //get os info and write system info
    NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:@"/System/Library/CoreServices/SystemVersion.plist"];
    NSString *osVersion = [dict objectForKey:@"ProductVersion"];
    NSString *system_build_number = [dict objectForKey:@"ProductBuildVersion"];
    NSString *osFamily = @"Mac OS";
    NSString *osGroup;
    NSString *osName;
    
    int version = [[[osVersion componentsSeparatedByString:@"."] objectAtIndex:1] intValue];
    switch (version)
    {
        case 0:
            osName = @"Cheetah";
            osGroup = @"OS X";
            break;
        case 1:
            osName = @"Puma";
            osGroup = @"OS X";
            break;
        case 2:
            osName = @"Jaguar";
            osGroup = @"OS X";
            break;
        case 3:
            osName = @"Panther";
            osGroup = @"OS X";
            break;
        case 4:
            osName = @"Tiger";
            osGroup = @"OS X";
            break;
        case 5:
            osName = @"Leopard";
            osGroup = @"OS X";
            break;
        case 6:
            osName = @"Snow Leopard";
            osGroup = @"OS X";
            break;
        case 7:
            osName = @"Lion";
            osGroup = @"OS X";
            break;
        case 8:
            osName = @"Mountain Lion";
            osGroup = @"OS X";
            break;
        default:
            osName = @"Error in Script";
            osGroup = @"Error in Script";
            break;
    }
		
    //<system> rootElement
    NSXMLElement *system = [[NSXMLElement alloc] initWithName:@"system"];
    
    //parent node of system_info
    NSXMLElement *sys = [[NSXMLElement alloc] initWithName:@"sys"];
    [system addChild:sys];
        
        NSMutableDictionary *network_info = [NSMutableDictionary dictionary];
        for (id key in @[@"system_timestamp", @"system_uuid", @"system_hostname", @"system_domain", @"system_type", @"system_os_icon", @"system_os_group", @"system_os_family", @"system_os_name", @"system_os_version", @"system_serial", @"system_model", @"system_manufacturer", @"system_uptime", @"system_form_factor", @"system_pc_os_bit", @"system_pc_memory", @"system_pc_num_processor", @"system_pc_date_os_installation", @"man_org_id"]) {
            
            //system_info[key] = [[XmlObject alloc] initWithParent:sys atName:key];
            
            network_info[key] = [[xmlObject alloc] init]; //allocate and initialize each key as an xmlObject
            [network_info[key] parent:sys tagName:key]; //set parent and tag name
        }
        
        //((xmlObject)system_info[@"pp"]).xmlElement = 0;
        //system.parent = sys;
        
        //Set value of each XML tag
        [network_info[@"system_timestamp"] setValue: theDate];
        [network_info[@"system_uuid"] setValue: system_uuid];
        [network_info[@"system_hostname"] setValue: hostName]; //wrong variable
        [network_info[@"system_domain"] setValue: @""];
        [network_info[@"system_description"]  setValue: @""];
        [network_info[@"system_type"] setValue: @""];
        [network_info[@"system_os_icon"] setValue: @""];
        [network_info[@"system_os_group"] setValue: osGroup];
        [network_info[@"system_os_family"] setValue: osFamily];
        [network_info[@"system_os_name"] setValue: osName];
        [network_info[@"system_os_version"] setValue: osVersion];
        [network_info[@"system_serial"] setValue: systemSerial];
        [network_info[@"system_model"] setValue: systemModel];
        [network_info[@"system_manufacturer"] setValue: @"Apple Inc."];
        [network_info[@"system_uptime"] setValue: @""];
        [network_info[@"system_form_factor"] setValue: @""];
        [network_info[@"system_pc_os_bit"] setValue: osBit];
        [network_info[@"system_pc_memory"] setValue: pcMemString];
        [network_info[@"system_pc_num_processor"] setValue: @""];
        [network_info[@"system_pc_date_os_installation"] setValue: @""];
        [network_info[@"man_org_id"] setValue: @""];
        
        	//Need source info for these variables. All Hardcoded
         NSString *net_domain_role = @"";
         NSString *time_caption = @"";
         NSString *time_daylight = @"";
         NSString *system_country_code = @"1";
         NSString *system_boot_device = @"";
         NSString *system_organization = @"System Organization";
         NSString *system_language = @"English";
         NSString *system_registered_user = @"Registered User";
         NSString *system_service_pack = @"";
         NSString *system_windows_directory = @"";
         NSString *net_domain_controller_address = @"";
         NSString *net_domain_controller_name = @"";
         
        NSXMLElement *windows = [[NSXMLElement alloc] initWithName:@"windows"];
        [system addChild:windows];
        
        NSMutableDictionary *mac_info = [NSMutableDictionary dictionary];
        for (id key in @[@"windows_build_number", @"windows_user_name", @"windows_client_site_name", @"windows_domain_short", @"windows_domain_controller_address", @"windows_domain_controller_name", @"windows_domain_role", @"windows_part_of_domain", @"windows_id_number", @"windows_time_caption", @"windows_time_daylight", @"windows_boot_device", @"windows_country_code", @"windows_organisation", @"windows_language", @"windows_registered_user", @"windows_service_pack", @"windows_version", @"windows_install_directory", @"windows_active_directory"]) {
            
            mac_info[key] = [[xmlObject alloc] init]; //allocate and initialize each key as an xmlObject
            [mac_info[key] parent:windows tagName:key]; //set parent and tag name
        }
        
        //Set value of each XML tag
        [mac_info[@"windows_build_number"] setValue: @""];
        [mac_info[@"windows_user_name"] setValue: net_user_name];
        [mac_info[@"windows_client_site_name"] setValue: @""];
        [mac_info[@"windows_domain_short"] setValue: net_domain];
        [mac_info[@"windows_domain_controller_address"] setValue: net_domain_controller_address];
        [mac_info[@"windows_domain_controller_name"] setValue: net_domain_controller_name];
        [mac_info[@"windows_domain_role"] setValue: net_domain_role];
        [mac_info[@"windows_part_of_domain"] setValue: @""];
        [mac_info[@"windows_id_number"] setValue: @""];
        [mac_info[@"windows_time_caption"] setValue: time_caption];
        [mac_info[@"windows_time_daylight"] setValue: time_daylight];
        [mac_info[@"windows_boot_device"] setValue: system_boot_device];
        [mac_info[@"windows_country_code"] setValue: system_country_code];
        [mac_info[@"windows_organisation"] setValue: system_organization];
        [mac_info[@"windows_language"] setValue: system_language];
        [mac_info[@"windows_registered_user"] setValue: system_registered_user];
        [mac_info[@"windows_service_pack"] setValue: system_service_pack];
        [mac_info[@"windows_version"] setValue: osVersion];
        [mac_info[@"windows_install_directory"] setValue: system_windows_directory];
        [mac_info[@"windows_active_directory"] setValue: @""];
        
#pragma mark Network info
    
    //Network info
    outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml",  @"SPNetworkDataType",nil]];
    NSMutableArray *networkInfoArray = (NSMutableArray *)[NSPropertyListSerialization
                                                          propertyListFromData:outFileData
                                                          mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                                          format:NULL
                                                          errorDescription:&errorString];
    NSArray *networkCardsArray = [[networkInfoArray objectAtIndex:0] objectForKey:@"_items"];
    NSString *bsdName, *name, *macAddress, *ipAddress, *subnet, *isConnected, *isConnectedIP, *net_dhcp_server, *net_dns_server, *net_dns_domain;
		NSString *network_description, *net_dhcp_enabled, *wins_primary, *wins_secondary, *net_manufacturer, *net_gateway;
		
        NSXMLElement *network_cards = [[NSXMLElement alloc] initWithName:@"network_cards"];
        [system addChild:network_cards];
        
        NSXMLElement *addresses = [[NSXMLElement alloc] initWithName:@"addresses"];
        [system addChild:addresses];
        
    for (NSDictionary *currentNetworkCard in networkCardsArray)
    {
        bsdName = [currentNetworkCard objectForKey:@"interface"];
        name = [currentNetworkCard objectForKey:@"_name"];
        macAddress = [[currentNetworkCard objectForKey:@"Ethernet"] objectForKey:@"MAC Address"];
        if (!macAddress)
            macAddress = @"";
        ipAddress = [[[currentNetworkCard objectForKey:@"IPv4"] objectForKey:@"Addresses"] objectAtIndex:0];
        if (!ipAddress)
            ipAddress = @"";        
        subnet = [[[currentNetworkCard objectForKey:@"IPv4"] objectForKey:@"SubnetMasks"] objectAtIndex:0];
        if (!subnet)
            subnet = @"";
			
        isConnected = [currentNetworkCard objectForKey:@"ip_assigned"];
        if ([isConnected isEqualToString:@"yes"])
				{
					isConnected = @"Connected";
					isConnectedIP = ipAddress;
				}
        else 
            isConnected = @"Not Connected";

			net_dhcp_server = [[currentNetworkCard objectForKey:@"dhcp"] objectForKey:@"dhcp_server_identifier"];
			net_dns_server = [[[currentNetworkCard objectForKey:@"DNS"] objectForKey:@"ServerAddresses"] objectAtIndex:0];
			net_dns_domain = [[currentNetworkCard objectForKey:@"DNS"] objectForKey:@"DomainName"];
			
			//Need source info for these variables.
			network_description = @"Network Description";
			net_dhcp_enabled = @"";
			wins_primary = @"";
			wins_secondary = @"";
			net_manufacturer = @"Apple Inc.";
			net_gateway = @"";
        
        NSXMLElement *network_card = [[NSXMLElement alloc] initWithName:@"network_card"];
        [network_cards addChild:network_card];
        
        NSMutableDictionary *network_info = [NSMutableDictionary dictionary];
        for (id key in @[@"net_mac_address", @"net_manufacturer", @"net_model", @"net_description", @"net_ip_enabled", @"net_index", @"net_connection_id", @"net_connection_status", @"net_speed", @"net_adapter_type", @"net_dhcp_enabled", @"net_dhcp_server", @"net_dhcp_lease_obtained", @"net_dhcp_lease_expires", @"net_dns_host_name", @"net_dns_domain", @"net_dns_server", @"net_wins_primary", @"net_wins_secondary", @"net_wins_lmhosts_enabled", @"ip_address", @"subnet"]) {
            
            network_info[key] = [[xmlObject alloc] init]; //allocate and initialize each key as an xmlObject
            [network_info[key] parent:network_card tagName:key]; //set parent and tag name
        }
        
        [network_info[@"net_mac_address"] setValue: macAddress];
        [network_info[@"net_manufacturer"] setValue: net_manufacturer];
        [network_info[@"net_model"] setValue: name];
        [network_info[@"net_description"] setValue: network_description];
        [network_info[@"net_ip_enabled"]  setValue: @""];
        [network_info[@"net_index"] setValue: @""];
        [network_info[@"net_connection_id"] setValue: @""];
        [network_info[@"net_connection_status"] setValue: isConnected];
        [network_info[@"net_speed"] setValue: @""];
        [network_info[@"net_adapter_type"] setValue: @""];
        [network_info[@"net_dhcp_enabled"] setValue: net_dhcp_enabled];
        [network_info[@"net_dhcp_server"] setValue: net_dhcp_server];
        [network_info[@"net_dhcp_lease_obtained"] setValue: @""];
        [network_info[@"net_dhcp_lease_expires"] setValue: @""];
        [network_info[@"net_dns_host_name"] setValue: hostName];
        [network_info[@"net_dns_domain"] setValue: net_dns_domain];
        [network_info[@"net_dns_server"] setValue: net_dns_server];
        [network_info[@"net_wins_primary"] setValue: wins_primary];
        [network_info[@"net_wins_secondary"] setValue: wins_secondary];
        [network_info[@"net_wins_lmhosts_enabled"] setValue: @""];
            /* Unused variables: net_dns_server_2, ipAddress, subnet, net_gateway, bsdName*/
        
        NSXMLElement *ip_address = [[NSXMLElement alloc] initWithName:@"ip_address"];
        [addresses addChild:ip_address];
        
        NSMutableDictionary *address_info = [NSMutableDictionary dictionary];
        for (id key in @[@"net_mac_address", @"ip_address_v4", @"ip_address_v6", @"ip_subnet", @"ip_address_version"]) {
            
            address_info[key] = [[xmlObject alloc] init]; //allocate and initialize each key as an xmlObject
            [address_info[key] parent:ip_address tagName:key]; //set parent and tag name
        }
        
        [address_info[@"net_mac_address"] setValue: macAddress];
        [address_info[@"ip_address_v4"] setValue: ipAddress];
        [address_info[@"ip_address_v6"] setValue: @""];
        [address_info[@"ip_subnet"] setValue: subnet];
        [address_info[@"ip_address_version"]  setValue: @""];
    }
    /*
    //kernel version
    outFileData =[taskLauncher launchTask: @"/usr/bin/uname" withArguments: [NSArray arrayWithObject: @"-r"]];
    outFileString = [[NSString alloc] initWithData:outFileData encoding:NSUTF8StringEncoding];
    NSString *kernelVersion = [outFileString stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];

    //kernel name
    outFileData =[taskLauncher launchTask: @"/usr/bin/uname" withArguments: [NSArray arrayWithObject: @"-v"]];
    outFileString = [[NSString alloc] initWithData:outFileData encoding:NSUTF8StringEncoding];
    NSString *kernelName = [outFileString stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
    
    //kernel type
    outFileData =[taskLauncher launchTask: @"/usr/bin/uname" withArguments: [NSArray arrayWithObject: @"-p"]];
    outFileString = [[NSString alloc] initWithData:outFileData encoding:NSUTF8StringEncoding];
    // NSString *kernelType = [outFileString stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
     */   
#pragma mark BootROM info
    
        //bootROM version    
        NSString *bootROMVersion = [hwItems objectForKey:@"boot_rom_version"];
        
        NSXMLElement *bios = [[NSXMLElement alloc] initWithName:@"bios"];
        [system addChild:bios];
        
        NSMutableDictionary *bios_info = [NSMutableDictionary dictionary];
        for (id key in @[@"bios_description", @"bios_manufacturer", @"bios_serial", @"bios_smversion", @"bios_version"]) {
            //system_info[key] = [[XmlObject alloc] initWithParent:sys atName:key];
            bios_info[key] = [[xmlObject alloc]init];
            [bios_info[key] parent:bios tagName:key];
        }
        
        [bios_info[@"bios_description"] setValue: bootROMVersion];
        [bios_info[@"bios_manufacturer"] setValue: @"Apple Inc."];
        [bios_info[@"bios_serial"] setValue: @""];
        [bios_info[@"bios_smversion"]  setValue: @""];
        [bios_info[@"bios_version"] setValue: @""];
        //((xmlObject)system_info[@"pp"]).xmlElement = 0;
        
        //system.parent = sys;
   
#pragma mark CPU info
        
        //CPU info
        NSArray *cpuArray = [[hwArray objectAtIndex:0] objectForKey:@"_items"];
        NSString *cpuSpeed;
        NSString *cpuName;
        NSString *cpuNumberProcessors;
        NSString *cpuNumberCores;
        NSString *cpuManufacturer = @"Intel Corporation";

        for (NSDictionary *currentCPU in cpuArray)
        {
            cpuSpeed = [currentCPU objectForKey:@"current_processor_speed"];
            cpuName = [currentCPU objectForKey:@"cpu_type"];
            cpuNumberProcessors = [currentCPU objectForKey:@"packages"];
            cpuNumberCores = [currentCPU objectForKey:@"number_processors"];
            int speed = (int)([cpuSpeed floatValue] * 1000);
            cpuSpeed = [NSString stringWithFormat:@"%d", speed];
            
            NSXMLElement *processor = [[NSXMLElement alloc] initWithName:@"processor"];
            [system addChild: processor];
            
            NSMutableDictionary *processor_info = [NSMutableDictionary dictionary];
            for (id key in @[@"processor_cores", @"processor_socket", @"processor_description", @"processor_speed", @"processor_manufacturer", @"processor_power_management_supported"]) {
                processor_info[key] = [[xmlObject alloc] init];
                [processor_info[key] parent:processor tagName:key];
            }
            
            [processor_info[@"processor_cores"] setValue: cpuNumberCores];
            [processor_info[@"processor_socket"] setValue: @""];
            [processor_info[@"processor_description"] setValue: cpuName];
            [processor_info[@"processor_speed"] setValue: cpuSpeed];
            [processor_info[@"processor_manufacturer"] setValue: cpuManufacturer];
            [processor_info[@"processor_power_management_supported"] setValue: @""];
        }
        
#pragma mark Memory info
    
    //Memory info
    outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml",  @"SPMemoryDataType",nil]];
    NSMutableArray *memoryArray = (NSMutableArray *)[NSPropertyListSerialization
                                                 propertyListFromData:outFileData
                                                 mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                                 format:NULL
                                                 errorDescription:&errorString];
    NSArray *memBanksArray = [[[[memoryArray objectAtIndex:0] objectForKey:@"_items"] objectAtIndex:0] objectForKey:@"_items"];
    if ([memBanksArray count] == 0)
     memBanksArray = [[memoryArray objectAtIndex:0] objectForKey:@"_items"];
    NSString *memName, *memType, *memSize, *memSpeed;
    NSString *nameS, *ddrType, *sync, *speedS;
    NSRange range;
        
        NSXMLElement *mem = [[NSXMLElement alloc] initWithName:@"memory"];
        [system addChild:mem];
        
    for (NSDictionary *currentMemBank in memBanksArray)
    {
        memName = [currentMemBank objectForKey:@"_name"];
        memType = [currentMemBank objectForKey:@"dimm_type"];
        memSize = [currentMemBank objectForKey:@"dimm_size"];
        memSpeed = [currentMemBank objectForKey:@"dimm_speed"];
        
        NSArray *memSizeList = [memSize componentsSeparatedByString:@" "];
        int memSizeInt = [memSizeList[0] intValue];
        memSizeInt = memSizeInt * 1024;
        NSString *memSize = [NSString stringWithFormat:@"%d", memSizeInt];
        
        unichar memN = [memName characterAtIndex:([memName length]-7)];//[[[[memName componentsSeparatedByString:@"/"] objectAtIndex:0] componentsSeparatedByString:@" "] objectAtIndex:1];

        nameS = [NSString stringWithFormat:@"DIMM_%C", memN];
        
        ddrType = [[memType componentsSeparatedByString:@" "] objectAtIndex:0];
        if ([[memType componentsSeparatedByString:@" "] count] > 1)
        {
            range = [[[[memType componentsSeparatedByString:@" "] objectAtIndex:1] lowercaseString] rangeOfString:@"s"];
            if (range.location != NSNotFound)
                sync = @"Synchronous";
            else
                sync = @"Asynchronous";
        }
        else
            sync = @"Asynchronous";
        
        float cSpeed = round(1.0 / [memSpeed floatValue] * 10000.0) / 10.0;
        speedS = [NSString stringWithFormat:@"%@ (%1.1f ns)", memSpeed, cSpeed];
        
        NSXMLElement *slot = [[NSXMLElement alloc] initWithName:@"slot"];
        [mem addChild:slot];
        
        NSMutableDictionary *mem_info = [NSMutableDictionary dictionary];
        for (id key in @[@"bank", @"type", @"form_factor", @"detail", @"capacity", @"speed", @"tag"]) {
            //system_info[key] = [[XmlObject alloc] initWithParent:sys atName:key];
            mem_info[key] = [[xmlObject alloc]init];
            [mem_info[key] parent:slot tagName:key];
        }
        
        [mem_info[@"bank"] setValue: nameS];
        [mem_info[@"type"] setValue: sync];
        [mem_info[@"form_factor"] setValue: @"DIMM"];
        [mem_info[@"detail"]  setValue: ddrType];
        [mem_info[@"capacity"] setValue: memSize];
        [mem_info[@"speed"] setValue: memSpeed];
        [mem_info[@"tag"] setValue: @""];
    }

#pragma mark Video Cards    

    //Video card(s) info
    outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml",  @"SPDisplaysDataType",nil]];
    NSMutableArray *videoInfoArray = (NSMutableArray *)[NSPropertyListSerialization
                                                     propertyListFromData:outFileData
                                                     mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                                     format:NULL
                                                     errorDescription:&errorString];
    NSArray *videoCardsArray = [[videoInfoArray objectAtIndex:0] objectForKey:@"_items"];
    NSString *gpuMemory, *gpuVendor, *gpuModel, *gpuROMRevision;
        
    NSXMLElement *video_cards = [[NSXMLElement alloc] initWithName:@"video_cards"];
    [system addChild: video_cards];
        
    for (NSDictionary *currentVideoCard in videoCardsArray)
    {
        gpuMemory = [currentVideoCard objectForKey:@"spdisplays_vram"];
        gpuVendor = [currentVideoCard objectForKey:@"spdisplays_vendor"];
        gpuModel = [currentVideoCard objectForKey:@"sppci_model"];
        gpuROMRevision = [currentVideoCard objectForKey:@"spdisplays_rom-revision"];
        if (!gpuROMRevision)
            gpuROMRevision = @"";
        
        NSXMLElement *video_card = [[NSXMLElement alloc] initWithName:@"video_card"];
        [video_cards addChild: video_card];
        
        NSMutableDictionary *video_info = [NSMutableDictionary dictionary];
        for (id key in @[@"video_description", @"video_manufacturer", @"video_memory"]) {
            
            video_info[key] = [[xmlObject alloc] init];
            [video_info[key] parent:video_card tagName:key];
        }
        
        [video_info[@"video_description"] setValue: gpuModel];
        [video_info[@"video_manufacturer"] setValue: gpuVendor];
        [video_info[@"video_memory"] setValue: gpuMemory];
    }
        
#pragma mark Sound cards
    
    //Sound card(s) info
    outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml",  @"SPAudioDataType",nil]];
    NSMutableArray *audioInfoArray = (NSMutableArray *)[NSPropertyListSerialization
                                                        propertyListFromData:outFileData
                                                        mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                                        format:NULL
                                                        errorDescription:&errorString];
    NSArray *soundCardsArray = [[audioInfoArray objectAtIndex:0] objectForKey:@"_items"];
    NSString *scName;
        
        NSXMLElement *sound_cards = [[NSXMLElement alloc] initWithName:@"sound_cards"];
        [system addChild: sound_cards];
        
    for (NSDictionary *currentSoundCard in soundCardsArray)
    {
        scName = [currentSoundCard objectForKey:@"_name"];
        
        NSXMLElement *sound_card = [[NSXMLElement alloc] initWithName:@"sound_card"];
        [sound_cards addChild: sound_card];
        
        NSMutableDictionary *sound_info = [NSMutableDictionary dictionary];
        for (id key in @[@"sound_name", @"sound_manufacturer", @"sound_device_id"]) {
            
            sound_info[key] = [[xmlObject alloc] init];
            [sound_info[key] parent:sound_card tagName:key];
        }
        
        [sound_info[@"sound_name"] setValue: scName];
        [sound_info[@"sound_manufacturer"] setValue: @""];
        [sound_info[@"sound_device_id"] setValue: @""];
    }
    
#pragma mark Printers info
        
        //Printers info
        outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml",  @"SPPrintersDataType",nil]];
        NSMutableArray *printersInfoArray = (NSMutableArray *)[NSPropertyListSerialization
                                                               propertyListFromData:outFileData
                                                               mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                                               format:NULL
                                                               errorDescription:&errorString];
        NSArray *printersArray = [[printersInfoArray objectAtIndex:0] objectForKey:@"_items"];
        NSString *printerName, *printerDriverVersion, *printerURI;
        
        NSXMLElement *printers = [[NSXMLElement alloc] initWithName:@"printers"];
        [system addChild:printers];
        
        for (NSDictionary *currentPrinter in printersArray)
        {
            printerName = [currentPrinter objectForKey:@"_name"];
            printerDriverVersion = [currentPrinter objectForKey:@"driverversion"];
            printerURI = [currentPrinter objectForKey:@"uri"];
            
            NSXMLElement *printer = [[NSXMLElement alloc] initWithName:@"printer"];
            [printers addChild: printer];
            
            NSMutableDictionary *printer_info = [NSMutableDictionary dictionary];
            for (id key in @[@"man_ip_address", @"hostname", @"system_key", @"uuid", @"description", @"icon", @"model", @"manufacturer", @"printer_port_name", @"printer_shared", @"printer_shared_name", @"printer_location", @"printer_color", @"printer_duplex", @"printer_type", @"printer_name", @"printer_connection_status"]) {
                
                printer_info[key] = [[xmlObject alloc] init];
                [printer_info[key] parent:printer tagName:key];
            }
            
            [printer_info[@"man_ip_address"] setValue: @""];
            [printer_info[@"hostname"] setValue: @""];
            [printer_info[@"system_key"] setValue: @""];
            [printer_info[@"uuid"] setValue: @""];
            [printer_info[@"description"] setValue: printerDriverVersion]; //Driver Version in Description
            [printer_info[@"icon"] setValue: @""];
            [printer_info[@"model"] setValue: @""];
            [printer_info[@"manufacturer"] setValue: @""];
            [printer_info[@"printer_port_name"] setValue: @""];
            [printer_info[@"printer_shared"] setValue: @""];
            [printer_info[@"printer_shared_name"] setValue: @""];
            [printer_info[@"printer_location"] setValue: @""];
            [printer_info[@"printer_color"] setValue: @""];
            [printer_info[@"printer_duplex"] setValue: @""];
            [printer_info[@"printer_type"] setValue: @""];
            [printer_info[@"printer_name"] setValue: printerName];
            [printer_info[@"printer_connection_status"] setValue: printerURI]; //What is printer URI?
        }
        
#pragma mark Harddrives
        
        //Harddrives, optical drives and volumes info
        NSMutableArray *volumesArray = [[NSMutableArray alloc] initWithCapacity:10];
        
        outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml",  @"SPSerialATADataType",nil]];
        NSMutableArray *hddInfoArray = (NSMutableArray *)[NSPropertyListSerialization
                                                          propertyListFromData:outFileData
                                                          mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                                          format:NULL
                                                          errorDescription:&errorString];
        NSArray *hddArray = [[hddInfoArray objectAtIndex:0] objectForKey:@"_items"];
        NSArray *drivesOnController;
        NSString *driveModel, *driveSerialNumber, *driveBSDName;
        double driveSizeInBytes, volumeSize, freeSize;
        NSString *volumeString, *volumeName, *volumeFS, *volumeMountPoint, *partitionSize, *partitionFreeSpace, *partitionUsedSpace, *deviceID;
        
        NSMutableArray *hardDrivesArray = [[NSMutableArray alloc] initWithCapacity:10];
        // NSMutableArray *opticalDrivesArray = [[NSMutableArray alloc] initWithCapacity:10];
        
        NSXMLElement *hard_disks = [[NSXMLElement alloc] initWithName:@"hard_disks"];
        [system addChild:hard_disks];
        
        NSXMLElement *partitions = [[NSXMLElement alloc] initWithName:@"partitions"];
        [system addChild:partitions];
        
        for (NSDictionary *currentHDDController in hddArray)
        {
            int hardDriveIndex = 0;
            NSString *hdInd;
            drivesOnController = [currentHDDController objectForKey:@"_items"];
            for (NSDictionary *currentHDD in drivesOnController)
            {
                hardDriveIndex++;
                hdInd = [NSString stringWithFormat:@"%d", hardDriveIndex];
                driveSizeInBytes = 0;
                driveModel = [[currentHDD objectForKey:@"device_model"] stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
                driveSerialNumber = [[currentHDD objectForKey:@"device_serial"] stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
                driveSizeInBytes = [[currentHDD objectForKey:@"size_in_bytes"] doubleValue];
                driveBSDName = [currentHDD objectForKey:@"bsd_name"];

                NSString *driveSize = [NSString stringWithFormat:@"%d", (int)(driveSizeInBytes / (1048576))];
                
                //  if (driveSizeInBytes)
                NSXMLElement *hard_disk = [[NSXMLElement alloc] initWithName:@"hard_disk"];
                [hard_disks addChild: hard_disk];
                
                NSMutableDictionary *hard_disk_info = [NSMutableDictionary dictionary];
                for (id key in @[@"hard_drive_caption", @"hard_drive_index", @"hard_drive_interface_type", @"hard_drive_manufacturer", @"hard_drive_model", @"hard_drive_serial", @"hard_drive_size", @"hard_drive_device_id", @"hard_drive_partitions", @"hard_drive_status", @"hard_drive_scsi_logical_unit"]) {
                    
                    hard_disk_info[key] = [[xmlObject alloc] init];
                    [hard_disk_info[key] parent:hard_disk tagName:key];
                }
                
                [hard_disk_info[@"hard_drive_caption"] setValue: @""];
                [hard_disk_info[@"hard_drive_index"] setValue: hdInd];
                [hard_disk_info[@"hard_drive_interface_type"] setValue: @""];
                [hard_disk_info[@"hard_drive_manufacturer"] setValue: @""];
                [hard_disk_info[@"hard_drive_model"] setValue: driveModel];
                [hard_disk_info[@"hard_drive_serial"] setValue: driveSerialNumber];
                [hard_disk_info[@"hard_drive_size"] setValue: driveSize];
                [hard_disk_info[@"hard_drive_device_id"] setValue: @""];
                [hard_disk_info[@"hard_drive_partitions"] setValue: @""];
                [hard_disk_info[@"hard_drive_status"] setValue: @""];
                [hard_disk_info[@"hard_drive_scsi_logical_unit"] setValue: @""];
                
#pragma mark  Volumes (partitions)                
                
               for (NSDictionary *currentVolume in [currentHDD objectForKey:@"volumes"])
                 {
                     volumeName = [currentVolume objectForKey:@"_name"];
                     volumeFS = [currentVolume objectForKey:@"file_system"];
                     volumeSize = [[currentVolume objectForKey:@"size_in_bytes"] floatValue]/1048576;
                     freeSize = [[currentVolume objectForKey:@"free_space_in_bytes"] floatValue]/1048576;
                     volumeMountPoint = [currentVolume objectForKey:@"mount_point"];
                     deviceID = [currentVolume objectForKey:@"volume_uuid"];
                     
                     partitionSize = [NSString stringWithFormat:@"%d", (int)volumeSize];
                     partitionFreeSpace = [NSString stringWithFormat:@"%d", (int)freeSize];
                     partitionUsedSpace = [NSString stringWithFormat:@"%d", (int)(volumeSize-freeSize)];
                 
                     NSXMLElement *partition = [[NSXMLElement alloc] initWithName:@"partition"];
                     [partitions addChild: partition];
                     
                     NSMutableDictionary *partition_info = [NSMutableDictionary dictionary];
                     for (id key in @[@"hard_drive_index", @"partition_mount_type", @"partition_mount_point", @"partition_name", @"partition_size", @"partition_free_space", @"partition_used_space", @"partition_format", @"partition_caption", @"partition_device_id", @"partition_disk_index", @"partition_bootable", @"partition_type", @"partition_quotas_supported", @"partition_quotas_enabled", @"partition_serial"]) {
                         
                         partition_info[key] = [[xmlObject alloc] init];
                         [partition_info[key] parent:partition tagName:key];
                     }
                     
                     [partition_info[@"hard_drive_index"] setValue: @""];
                     [partition_info[@"partition_mount_type"] setValue: @""];
                     [partition_info[@"partition_mount_point"] setValue: volumeMountPoint];
                     [partition_info[@"partition_name"] setValue: volumeName];
                     [partition_info[@"partition_size"] setValue: partitionSize];
                     [partition_info[@"partition_free_space"] setValue: partitionFreeSpace];
                     [partition_info[@"partition_used_space"] setValue: partitionUsedSpace];
                     [partition_info[@"partition_format"] setValue: volumeFS];
                     [partition_info[@"partition_caption"] setValue: volumeMountPoint];
                     [partition_info[@"partition_device_id"] setValue: @""]; //deviceID
                     [partition_info[@"partition_disk_index"] setValue: @""];
                     [partition_info[@"partition_bootable"] setValue: @""];
                     [partition_info[@"partition_type"] setValue: @""];
                     [partition_info[@"partition_quotas_supported"] setValue: @""];
                     [partition_info[@"partition_quotas_enabled"] setValue: @""];
                     [partition_info[@"partition_serial"] setValue: @""];
                 }
            }
        }
        /*
#pragma mark Software info
    
    //Software info
    outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml", @"SPApplicationsDataType",nil]];
    NSMutableArray *appsInfoArray = (NSMutableArray *)[NSPropertyListSerialization
                                                       propertyListFromData:outFileData
                                                       mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                                       format:NULL
                                                       errorDescription:&errorString];
    NSArray *appsArray = [[appsInfoArray objectAtIndex:0] objectForKey:@"_items"];
    NSString *appName, *appVersion, *appInfo, *appPath;
        
        NSXMLElement *package = [[NSXMLElement alloc] initWithName:@"package"];
        [system addChild:package];

        
    for (NSDictionary *currentApp in appsArray)
    {
        appName = [currentApp objectForKey:@"_name"];
        appPath = [currentApp objectForKey:@"path"];
        appVersion = [currentApp objectForKey:@"version"];
        if (!appVersion)
            appVersion = @"";
        appInfo = [currentApp objectForKey:@"info"];
        if (!appInfo)
            appInfo = @"";
        
        NSXMLElement *software = [[NSXMLElement alloc] initWithName:@"software"];
        [package addChild: software];
        
        NSMutableDictionary *system_info = [NSMutableDictionary dictionary];
        for (id key in @[@"software_name", @"software_version", @"software_location", @"software_uninstall", @"software_install_date", @"software_publisher", @"software_install_source", @"software_system_component", @"software_url", @"software_email", @"software_comment", @"software_code_base", @"software_status", @"software_installed_by", @"software_installed_on"]) {
            
            system_info[key] = [[xmlObject alloc] init];
            [system_info[key] parent:software tagName:key];
        }
        
        [system_info[@"software_name"] setValue: appName];
        [system_info[@"software_version"] setValue: appVersion];
        [system_info[@"software_location"] setValue: appPath];
        [system_info[@"software_uninstall"] setValue: @""];
        [system_info[@"software_install_date"] setValue: @""];
        [system_info[@"software_publisher"] setValue: appInfo];
        [system_info[@"software_install_source"] setValue: @""];
        [system_info[@"software_system_component"] setValue: @""];
        [system_info[@"software_url"] setValue: @""];
        [system_info[@"software_email"] setValue: @""];
        [system_info[@"software_comment"] setValue: @""];
        [system_info[@"software_code_base"] setValue: @""];
        [system_info[@"software_status"] setValue: @""];
        [system_info[@"software_installed_by"] setValue: @""];
        [system_info[@"software_installed_on"] setValue: @""];
    }
        //((xmlObject)system_info[@"pp"]).xmlElement = 0;
/*
#pragma mark Optical drives info
    outFileData = [taskLauncher launchTask:@"/usr/sbin/system_profiler" withArguments:[NSArray arrayWithObjects: @"-xml",  @"SPDiscBurningDataType",nil]];

    NSMutableArray *opticalInfoArray = (NSMutableArray *)[NSPropertyListSerialization
                                                      propertyListFromData: outFileData//[ss dataUsingEncoding:NSUTF8StringEncoding]  //outFileData
                                                      mutabilityOption:NSPropertyListMutableContainersAndLeaves
                                                      format:NULL
                                                      errorDescription:&errorString];
    NSArray *opticalArray = [[opticalInfoArray objectAtIndex:0] objectForKey:@"_items"];
    NSString *opticalName;
    for (NSDictionary *currentOptical in opticalArray)
    {
        opticalName = [currentOptical objectForKey:@"_name"];
        
        stringToWrite = [NSString stringWithFormat:@"optical%@%@%@ %@ %@\n", 
							 sep, opticalName, 
							 sep, 
							 sep, 
							 sep];
        NSLog(@"%@", stringToWrite);
    }
   
#pragma mark ip routes
    
    NSArray *rows = [outFileString componentsSeparatedByString:@"\n"];
    NSArray *rowComponents;
    NSMutableArray *_elements;
        
    for (NSString *currentRow in rows)
    {
        rowComponents = [currentRow componentsSeparatedByString:@" "];
        _elements = [[NSMutableArray alloc] initWithCapacity:[rowComponents count]];
        for (NSString *s in rowComponents)
        {
            if (![s isEqualToString:@""])
                [_elements addObject:s];
        }
        
        if ([_elements count] >= 6)
        {
            if (![[_elements objectAtIndex:0] isEqualToString:@"Destination"])                
            {
                
                stringToWrite = [NSString stringWithFormat:@"ip_route%@%@%@%@%@0%@ %@%@%@%@\n", sep, [_elements objectAtIndex:0], sep, [_elements objectAtIndex:1], sep, sep, sep, sep, [_elements objectAtIndex:5], sep];
                NSLog(@"%@", stringToWrite);
                [outFile writeData:[stringToWrite dataUsingEncoding:NSUTF8StringEncoding]];
            }
        }
        
        _elements = nil;
    }

#pragma mark users info

    //get users info
    outFileData =[taskLauncher launchTask: @"/usr/bin/dscacheutil" withArguments: [NSArray arrayWithObjects: @"-q", @"user", nil]];
    outFileString = [[NSString alloc] initWithData:outFileData encoding:NSUTF8StringEncoding];
        
    NSArray *usersArray = [outFileString componentsSeparatedByString:@"\n\n"];
    NSArray *userComponents, *parts;
    NSString *uName, *uFullName, *uUID;
    for (NSString *fullUserInfo in usersArray)
    {
        userComponents = [fullUserInfo componentsSeparatedByString:@"\n"];
        for (NSString *comp in userComponents)
        {
            parts = [comp componentsSeparatedByString:@": "];
            if ([[parts objectAtIndex:0] isEqualToString:@"name"])
                uFullName = [parts objectAtIndex:1];
            else if ([[parts objectAtIndex:0] isEqualToString:@"uid"])
                uUID = [parts objectAtIndex:1];
            else if ([[parts objectAtIndex:0] isEqualToString:@"gecos"])
                uName = [parts objectAtIndex:1];
        }
        if ([userComponents count] > 1)
        {
            stringToWrite = [NSString stringWithFormat:@"l_user%@ %@ No %@ %@ %@ %@ %@True %@%@YES%@%@%@\n", sep, sep, sep, uFullName, sep, uName, sep, sep, sep, sep, uUID, sep];
            //l_user^^^ ^^^ No ^^^ root ^^^ System Administrator ^^^True ^^^^^^YES0^^^^^^
            NSLog(@"%@", stringToWrite);
            [outFile writeData:[stringToWrite dataUsingEncoding:NSUTF8StringEncoding]];
        }
    }
    
#pragma mark  group info
    
    //get group info
    outFileData =[taskLauncher launchTask: @"usr/bin/dscl" withArguments: [NSArray arrayWithObjects: nil, nil, nil]];
    outFileString = [[NSString alloc] initWithData:outFileData encoding:NSUTF8StringEncoding];
        NSLog(@"%@", outFileString);
  NSArray *groupsArray = [outFileString componentsSeparatedByString:@"\n\n"];
    NSArray *gComponents;
    NSString *gName, *gGID;
    //l_group^^^^^^at(name)^^^^^^25(gid)^^^
    for (NSString *fullGroupInfo in groupsArray)
    {
        gComponents = [fullGroupInfo componentsSeparatedByString:@"\n"];
        for (NSString *group in gComponents)
        {
            parts = [group componentsSeparatedByString:@": "];
            if ([[parts objectAtIndex:0] isEqualToString:@"name"])
                gName = [parts objectAtIndex:1];
            else if ([[parts objectAtIndex:0] isEqualToString:@"gid"])
                gGID = [parts objectAtIndex:1];
        }
        
        if ([gComponents count] > 1)
        {
            stringToWrite = [NSString stringWithFormat:@"l_group%@%@%@%@%@%@%@\n", sep, sep, gName, sep, sep, gGID, sep];
            NSLog(@"%@", stringToWrite);
            [outFile writeData:[stringToWrite dataUsingEncoding:NSUTF8StringEncoding]];
        }
    }*/
        
    //write to XML file using rootelement, set XML file version & characterEncoding
        
    NSXMLDocument *xmlRequest = [NSXMLDocument documentWithRootElement:system];
    [xmlRequest setVersion:@"1.0"];
    [xmlRequest setCharacterEncoding:@"UTF-8"];
    NSData *xmlData = [xmlRequest XMLDataWithOptions:NSXMLNodePrettyPrint];
    
    [xmlData writeToFile:outFileName atomically:YES];
    }
    return 0;

}